#!/usr/bin/env python
import setpath
import unittest
import os
from bike.testutils import trimLines,createAST
from bike.parsing.fastparser import fastparser
from bike.parsing.fastparserast import Module, Class
from common import indexToCoordinates, getScopeForLine

class TestGetScopeForLine(unittest.TestCase):

    def test_worksWithFunctionScope(self):
        src = trimLines("""
        class a:
            def foo():
                pass
        """)
        node = createAST(src)
        self.assertEqual(getScopeForLine(node,3).name,"foo")

    def test_worksWithModuleScope(self):
        src = trimLines("""
        class TheClass:
            pass
        a = TheClass()
        """)
        node = createAST(src)
        assert isinstance(getScopeForLine(node,3),Module)

    def test_worksWithInlineClass(self):
        src = trimLines("""
        class TheClass: pass""")
        node = createAST(src)
        assert isinstance(getScopeForLine(node,1),Class)


class TestIndexToCoordinates(unittest.TestCase):

    def test_worksOnSingleLineString(self):
        src = trimLines('''
        foo bah
        ''')
        x,y = indexToCoordinates(src,src.index("bah"))
        self.assertEqual(x,4)
        self.assertEqual(y,0)
        x,y = indexToCoordinates(src,src.index("foo"))
        self.assertEqual(x,0)
        self.assertEqual(y,0)

    def test_worksOnMultilLineString(self):
        src = trimLines('''
        foo bah
        baz boh
        ''')
        x,y = indexToCoordinates(src,src.index("boh"))
        self.assertEqual(x,4)
        self.assertEqual(y,1)


class TestTranslateSourceCoordsIntoASTNode(unittest.TestCase):

    def test_worksOnImport(self):
        from common import translateSourceCoordsIntoASTNode
        src = trimLines('''
        from foo import bar, baz
        ''')
        module = fastparser(src)
        node = translateSourceCoordsIntoASTNode(module, 0, 17)


if __name__ == "__main__":
    unittest.main()
