#!/usr/bin/env python
import setpath
import unittest
from rename import rename
from bike import testdata
from bike.testutils import*
from bike.parsing.extended_ast import Root
import compiler

class RenameFunctionTests:
    def runTarget(self, src, klassfqn, newname):
        # see concrete subclasses for implementation
        pass

    def testRenamesFunctionDcl(self):
        srcBefore=trimLines("""
        def theFunction():
            pass
        """)
        srcAfter=trimLines("""
        def newName():
            pass
        """)
        src = self.rename(srcBefore,1,4,"newName")
        self.assertEqual(srcAfter,src)

    def testDoesntBarfWhenFunctionIncludesBrackettedExpression(self):
        srcBefore=trimLines("""
        def theFunction():
            return ('\\n').strip()
        """)
        srcAfter=trimLines("""
        def newName():
            return ('\\n').strip()
        """)
        src = self.rename(srcBefore,1,4, "newName")
        self.assertEqual(srcAfter,src)




class RenameFunctionTests_importsFunction:

    def testRenamesImportedFunctionReference(self):
        srcBefore=trimLines("""
        import b.bah
        b.bah.theFunction()
        """)
        srcAfter=trimLines("""
        import b.bah
        b.bah.newName()
        """)
        src = self.renameFunction(srcBefore,"newName")
        self.assertEqual(srcAfter,src)

    def testRenamesFunctionReferenceImportedWithFromClause(self):
        srcBefore=trimLines("""
        from b.bah import theFunction
        theFunction()
        """)
        srcAfter=trimLines("""
        from b.bah import newName
        newName()
        """)
        src = self.renameFunction(srcBefore,"newName")
        self.assertEqual(srcAfter,src)

    def testRenamesFunctionRefInImportClause(self):
        srcBefore=trimLines("""
        import b.bah
        b.bah.theFunction()
        """)
        srcAfter=trimLines("""
        import b.bah
        b.bah.newName()
        """)
        src = self.renameFunction(srcBefore,"newName")
        self.assertEqual(srcAfter,src)


    def testRenamesFunctionRefInImportFromClause(self):
        srcBefore=trimLines("""
        from b.bah import theFunction
        theFunction()
        """)
        srcAfter=trimLines("""
        from b.bah import newName
        newName()
        """)
        src = self.renameFunction(srcBefore,"newName")
        self.assertEqual(srcAfter,src)



class TestRenameFunction(unittest.TestCase, RenameFunctionTests):
    def rename(self, src, line, col, newname):
        ast = createSourceNodeAt(Root(), src, "foo")
        rename(ast.getSource("foo"),
               line,col, newname)
        return ast.getSource("foo").getSource()


class TestRenameFunctionReferenceWithDirectoryStructure(unittest.TestCase, RenameFunctionTests_importsFunction):

    def renameFunction(self, src, newname):
        ast = Root()
        ast = createSourceNodeAt(ast, src, "a.foo")
        ast = createSourceNodeAt(ast, FunctionTestdata, "a.b.bah")
        sourcenode = getMatchingASTNode(ast, "Source",fqn="a.b.bah")
        rename(sourcenode, 1,4, newname)
        return getMatchingASTNode(ast, "Source",fqn="a.foo").getSource()

# checks that renameFunction can be done twice on the same ast
class TestRenameFunctionReferenceTwice(unittest.TestCase, RenameFunctionTests_importsFunction):

    def renameFunction(self, src, newname):
        ast = Root()
        ast = createSourceNodeAt(ast, src, "a.foo")
        ast = createSourceNodeAt(ast, FunctionTestdata, "a.b.bah")
        sourcenode = getMatchingASTNode(ast, "Source",fqn="a.b.bah")
        rename(sourcenode, 1,4, "AnotherName")
        rename(sourcenode, 1,4, newname)        
        return getMatchingASTNode(ast, "Source",fqn="a.foo").getSource()

FunctionTestdata = trimLines("""
def theFunction():
    pass
""")


if __name__ == "__main__":
    unittest.main()
