# Bicycle Repair Man integration with (X)Emacs
# By Phil Dawes (2002)
# Uses the fabulous Pymacs package by Franois Pinard

from Pymacs import lisp, Let
import bike
reload(bike)
from bike import bikefacade
import time

brmctx = bike.init()

is_xemacs = (lisp.emacs_version().find("GNU") == -1)
currently_saving=0

# fix pop_excursion to work with xemacs
class Let(Let):
    def pop_excursion(self):
        method, (buffer, point_marker, mark_marker) = self.stack[-1]
        assert method == 'excursion', self.stack[-1]
        del self.stack[-1]
        lisp.set_buffer(buffer)
        lisp.goto_char(point_marker)
        lisp.set_mark(mark_marker)
        lisp.set_marker(point_marker, None)
        if mark_marker is not None:   # needed for xemacs
            lisp.set_marker(mark_marker, None)

class EmacsLogger:
    def __init__(self):
        self.msg = ""
    
    def write(self,output):
        self.msg +=output
        if output.endswith("\n"):
            lisp.message(self.msg)
            self.msg = ""

class NullLogger:
    def write(self,output):
        pass
        
logger = EmacsLogger()


def init():
    lisp.require(lisp["python-mode"])
    lisp("""
    (defvar brm-menu nil "Menu for Bicycle Repair Man")
	 (easy-menu-define
      brm-menu py-mode-map "Bicycle Repair Man"
      '("BicycleRepairMan"
				   ["Import Code Into BRM" brm-load t]
				   "---"
				   "Queries"
				   ["Find-References" brm-find-references]
				   ["Find-Definition" brm-find-definition]
		           "---"
				   "Refactoring"
				   ["Rename" brm-rename t]
				   ["Extract-Method" brm-extract t]
				   ["Undo Last Refactoring" brm-undo t]

	))
	(add-hook 'python-mode-hook (lambda () (easy-menu-add brm-menu)))
    """)
    #lisp.add_hook(lisp.after_save_hook,lisp.brm_load_current)
    lisp.add_hook(lisp.find_file_hooks,lisp.brm_load_current)
    brmctx.setProgressLogger(logger)

def load(path):
    """
    Loads python source modules into Bicycle Repair Man
    Packages are loaded recursively
    """
    brmctx.load(path)
    print >>logger,"Done"
load.interaction='fPython file or directory: '

def load_current():
    if not currently_saving:
        filename = lisp.buffer_file_name()
        if filename.endswith(".py"):
            brmctx.load(filename)
    
def kill():
    """
    Removes the bicyclerepairman context (freeing the state),
    and reinitialises it.
    """
    global brmctx
    brmctx = None
    brmctx = bike.init()
    brmctx.setProgressLogger(logger)
kill.interaction=""


def promptCallback(filename,line,colbegin,colend):
    let = Let().push_excursion()   # gets popped when let goes out of scope
    buffer = lisp.current_buffer()
    if let:
        ans = 0
        lisp.find_file(filename)
        lisp.goto_line(line)
        lisp.move_to_column(colbegin)
        lisp.set_mark(lisp.point() + (colend - colbegin))
        if is_xemacs:
            lisp.activate_region()
        ans = lisp.y_or_n_p("Couldn't deduce object type - rename this method reference? ")
    del let
    lisp.switch_to_buffer(buffer)
    return ans

def rename(newname):
    lisp.save_some_buffers()
    filename = lisp.buffer_file_name()
    line,col = _getCoords()
    brmctx.setRenameMethodPromptCallback(promptCallback)
    try:
        brmctx.renameByCoordinates(filename,line,col,newname)
        savedFiles = brmctx.save()
        _revertSavedFiles(savedFiles)
        lisp.set_marker(lisp.mark_marker(),None)
    except bikefacade.CouldntLocateASTNodeFromCoordinatesException:
        print >>logger,"Couldn't find AST Node. Are you renaming the declaration?"
    
rename.interaction="sNew name: "


def undo():
    brmctx.undo()
    savedFiles = brmctx.save()
    _revertSavedFiles(savedFiles)
undo.interaction=""


def _revertSavedFiles(savedFiles):
    global currently_saving
    currently_saving = 1
    for file in savedFiles:
        buf = lisp.find_buffer_visiting(file)
        if buf:
            lisp.set_buffer(buf)
            lisp.revert_buffer(None,1)
    currently_saving = 0

def find_references():    
    lisp.save_some_buffers()
    filename = lisp.buffer_file_name()
    line,col = _getCoords()
    refs = brmctx.findReferencesByCoordinates(filename,line,col)
    _switchToConsole()
    numRefs = 0
    for ref in refs:
        _insertRefLineIntoConsole(ref)
        numRefs +=1
    lisp.insert("Done - %d refs found\n"%numRefs)
    
find_references.interaction=""

def _getCoords():
    line = lisp.count_lines(1,lisp.point())
    col = lisp.current_column()
    if col == 0:            
        line += 1  # get round 'if col == 0, then line is 1 out' problem

    if mark_exists() and lisp.point() > lisp.mark():
        lisp.exchange_point_and_mark()
        col = lisp.current_column()
        lisp.exchange_point_and_mark()
    return line,col

def mark_exists():
    if is_xemacs:
        return lisp.mark()
    else:
        return lisp("mark-active") and lisp.mark()



def _switchToConsole():
    consolebuf = lisp.get_buffer_create("BicycleRepairManConsole")
    lisp.switch_to_buffer_other_window(consolebuf)
    lisp.compilation_mode("BicycleRepairMan")
    lisp.erase_buffer()
    lisp.insert("Bicycle Repair Man\n")
    lisp.insert("(Hint: Press Return a Link)\n")
    
def find_definition():
    lisp.save_some_buffers()
    filename = lisp.buffer_file_name()
    line,col = _getCoords()
    defns = brmctx.findDefinitionByCoordinates(filename,line,col)

    try:
        firstdefn = defns.next()
        lisp.find_file_other_window(firstdefn.filename)
        lisp.goto_line(firstdefn.lineno)
        lisp.forward_char(firstdefn.colno)
    except StopIteration:
        pass
    else:
        numRefs = 1
        for defn in defns:
            if numRefs == 1:
                _switchToConsole()
                _insertRefLineIntoConsole(firstdefn)
            _insertRefLineIntoConsole(defn)
            numRefs += 1
find_definition.interaction=""

def _insertRefLineIntoConsole(ref):
    lisp.insert(ref.filename+":"+str(ref.lineno)+":    "+str(ref.confidence)+"% confidence\n")
    _redisplayFrame()

def _redisplayFrame():
    lisp.sit_for(0)

def extract(name):
    lisp.save_some_buffers()
    filename = lisp.buffer_file_name()

    bline,bcol = _getPointCoords()
    lisp.exchange_point_and_mark()
    eline,ecol = _getPointCoords()
    lisp.exchange_point_and_mark()

    brmctx.extract(filename,bline,bcol,eline,ecol,name)
    lisp.set_marker(lisp.mark_marker(),None)
    _revertSavedFiles(brmctx.save())
extract.interaction="sName of function: "

def _getPointCoords():
    bline = lisp.count_lines(1,lisp.point())
    bcol = lisp.current_column()
    if bcol == 0:  # get round line is one less if col is 0 problem
        bline += 1
    return bline,bcol

